package filter
{
	import flash.display.BitmapData;
	import flash.display.Shader;
	import flash.display.ShaderJob;
	
	public class FloretTilingFilter 
	{
		
		/** source image */
		public var	input:BitmapData;
			
		/**	destination image */
		public var	output:BitmapData;

		
		/**	shader */
		protected var shader:Shader;
		
		/**	embedded .pbj	*/
		[Embed(source='../assets/FloretPentagonal.pbj', mimeType = 'application/octet-stream')]
		protected var EmbeddedShader:Class;
		
		
		/**	Constructor */
		public function FloretTilingFilter(
			_input:BitmapData,
			_output:BitmapData)
		{
			input=_input;
			output=_output;
			shader = new Shader(new EmbeddedShader());
		}
		
		/**	Applies the shader to image */
		public function render():void {
			if (input==null) return;
			shader.data.src.input = input;
			var shaderJob:ShaderJob = new ShaderJob(shader, output, output.width, output.height);
			
			output.lock();
			//	apply the shader, true = waits here for completion
			shaderJob.start(true);
			output.unlock();
		}
		
		
		
		/**	Pattern size */
		public function set size (value:Number):void {
			shader.data.size.value = [value];
			render();
		}
		
		/**	Sets the pattern rotation angle, value in degrees */
		public function set rotation(degrees:Number):void { 
			var radians:Number = degrees*Math.PI/180,
				cs:Number = Math.cos(radians),
				sn:Number = Math.sin(radians);
			//	set the rotation matrix
			shader.data.rotation.value  = [ cs,sn,-sn,cs ];
			//	and the reverse matrix
			shader.data.rotationR.value = [ cs,-sn,sn,cs ];
			render();
		}
		
		/**	Base point for pattern origin, parameter point: any kind of object with properties x and y */
		public function set base(point:Object):void { 
			shader.data.base.value = [ point.x, point.y ]; 
			render();
		}
		
		/**	set fading between cells */
		public function set fade(value:Number):void {
			shader.data.fade.value= [value]; 
			render();
		}
		
	}
}